/**
 * @file main.c
 * @brief Este projeto demonstra o uso de um buffer circular para compatibilizar
 * a velocidade de dois dispositivos. Neste caso, interface serial e Kinetis.
 * @author Wu Shin-Ting
 * @date 25/11/2020
 *
 */

#include "derivative.h"

#include "stdlib.h"
//==================================
// Utilities
//==================================
/**
 * @brief atraso em numero de interacoes <i>
 * @param[in] i numero de iteracoes
 */
void delay (uint32_t i)
{
	__asm("outerLoop:;"
			"sub %0, %0, #1;"
			"mov r2, #11;"
			"innerLoop:;"
			"sub r2, r2, #1;"
			"cmp r2, #0;"
			"bne innerLoop;"
			"cmp %0, #0;"
			"bne outerLoop;"
			: 
			: "r" (i)
	);
}

//==================================
//Clocks
//==================================
/**
 * @brief Setar bus clock em 20,97152MHz/[OUTDIV4]
 */
void SIM_initBusClock (uint8_t OUTDIV4) {	
	SIM_CLKDIV1 &= ~SIM_CLKDIV1_OUTDIV4(0b111);  //resetar todos os bits
	SIM_CLKDIV1 |= SIM_CLKDIV1_OUTDIV4(OUTDIV4); //mascarar 1s
}

/**
 * @brief Selecionar a base de tempo para os clocks dos modulos do microcontrolador
 * @param[in] base (MCGFLLCLK=0; MCGPLLCLK=1)
 */
void SIM_selPLLFLL (uint8_t base) {
	SIM_SOPT2 &= ~SIM_SOPT2_PLLFLLSEL_MASK;
	SIM_SOPT2 |= (base << SIM_SOPT2_PLLFLLSEL_SHIFT) & SIM_SOPT2_PLLFLLSEL_MASK;
}

//==================================
// PORTx
//==================================
void PORTx_muxUART0RxTxPins() {
	SIM_SCGC5 |= SIM_SCGC5_PORTA_MASK;      // habilitar clock gate de PORTA
	PORTA_PCR1 |= PORT_PCR_MUX(0x2);        // configurar com funcao UART0_RX
	PORTA_PCR2 |= PORT_PCR_MUX(0x2);        // configurar com funcao UART0_TX
}	

//====================================
//  GPIO
//====================================
void GPIO_initLedRGB() {
	// Modulo SIM: habilita o clock do modulo PORTE
	SIM_SCGC5 |= SIM_SCGC5_PORTE_MASK; 

	// Modulo PORT: configure o pino PTE21 para GPIO (vermelho)
	PORTE_PCR21 |= PORT_PCR_MUX (0b001); // mascarar os bits 8, 9 e 10

	// Modulo PORT: configure o pino PTE22 para GPIO (verde)
	PORTE_PCR22 |= PORT_PCR_MUX (0b001); // mascarar os bits 8, 9 e 10

	// Modulo PORT: configure o pino PTE23 para GPIO (azul)
	PORTE_PCR23 |= PORT_PCR_MUX (0b001); // mascarar os bits 8, 9 e 10

	// Modulo GPIO: configure o sentido do sinal nos pinos PTE21, PTE22 e PTE23
	GPIOE_PDDR |= (0b111<<21);    // Seta pinos 21, 22 e 23 do PORTE como saida 	

	// Modulo GPIO: inicializar PTE21, PTE22 e PTE23 em 0 (estado default no reset)
	GPIOE_PCOR = (0b111<<21);    // Resetar pinos 21, 22 e 23 do PORTE
}
//==================================
// UART0
//==================================
/**
 * @Parametros de configuracao  UART0
 */
typedef struct _UART0Configuration_tag {
	uint8_t	bdh_sbns;		// selecionar quantidade de stop bits (0 = 1; 1 = 2)
	uint16_t sbr;		
	uint8_t c1_loops;       // operacao em loop (normal = 0)
	uint8_t c1_dozeen;		// habilitar espera (doze)
	uint8_t c1_rsrc;		// habilitar a saida do TX em operao de loop
	uint8_t c1_m; 			// 8-bit (0) ou 9-bit de dados
	uint8_t c1_wake;		// forma de wakeup do RX (0-idle line; 1-address-mark)
	uint8_t c1_ilt;			// selecionar a forma de deteccao de "idle line"
	uint8_t c1_pe; 			// habilitar paridade
	uint8_t c1_pt;			// tipo de paridade (0-par; 1-impar)
	uint8_t c2_rwu; 		// setar o RX no estado de standby aguardando pelo wakeup
	uint8_t c2_sbk;			// habilitar o enfileiramento de caracteres break
	uint8_t	s2_msbf;		// setar endianess para MSB (0 = LSB;1 = MSB)
	uint8_t s2_rxinv;		// habilitar a inversao da polaridade dos bits do RX
	uint8_t s2_rwuid;		// habilitar o set do IDLE bit durante standby do RX
	uint8_t s2_brk13;		// selecionar o comprimento de caracter break (0=10 bits; 1=13 bits)
	uint8_t s2_lbkde;		// habilitar deteccao de caractere break longo
	uint8_t c3_r8t9; 		// bit 8 (RX)/bit 9 (TX)
	uint8_t c3_r9t8;		// bit 9 (RX)/bit 8 (TX)
	uint8_t c3_txdir; 		// na configuracao RSRC=1, configurar o sentido de TX (0=entrada; 1=saida)
	uint8_t c3_txinv;		// habilitar a inversao da polaridade dos bits de TX
	uint8_t c4_maen1;		// habilitar controle de "match address" 1
	uint8_t c4_maen2; 		// habilitar controle de "match address" 2
	uint8_t c4_m10;			// selecionar o modo de bits (0=8/9 bits;1=10 bits)
	uint8_t c4_osr;			// taxa de super-amostragem (default=16(0b01111)
	uint8_t c5_tdmae; 		// habilitar transmissao por DMA
	uint8_t c5_rdmae; 		// habilitar recepcao por DMA
	uint8_t c5_bothedge; 	// amostrar os dados em ambas as bordas do clock de baud rate
	uint8_t c5_resyncdis;	// desabilitar o resincronismo nas transies de 1 para 0
} UART0Config_type;

typedef struct UART0ConfigInterrupts_tag {
	uint8_t c2_tie;		// habilitar interrupcao TX
	uint8_t c2_tcie;	// habilitar interrupcao completa TX
	uint8_t c2_rie;		// habilitar interrupcao do receptor
	uint8_t c2_ilie;	// habilitar interrupcao por linha ociosa
	uint8_t c3_orie;	// habilitar interrupcao por overrun
	uint8_t c3_neie;	// habilitar interrupcao por noise error
	uint8_t c3_feie;	// habilitar interrupcao por framing error
	uint8_t	c3_peie;	// habilitar interrupcao por parity error
} UART0ConfigInterrupts_type;

/**
 * @brief Inicializar os pinos conectados a UART0 e ao OpenSDA.
 * @return codigo de erro (1=taxa invalida; 2=baud invalido; 3=combinacao dos dois invalida)
 */
uint8_t UART0_init(UART0Config_type *config) {
	SIM_SCGC4 |= SIM_SCGC4_UART0_MASK;      // habilitar clock gate de UART0

	SIM_SOPT2 &= ~SIM_SOPT2_UART0SRC(0b11); // configurar a fonte de relgio (20.971520MHz)
	SIM_SOPT2 |= SIM_SOPT2_UART0SRC(0x1);	

	// Desabilitar canais RX e TX para configuracoes
	UART0_C2 &= ~(UART0_C2_RE_MASK |           
			UART0_C2_TE_MASK);

	// Configurar stop bits
	UART0_BDH &= ~(UART0_BDH_SBNS_MASK);
	UART0_BDH |= config->bdh_sbns << UART0_BDH_SBNS_SHIFT;

	// Configurar o registrador C1
	UART0_C1 &= ~(UART0_C1_LOOPS_MASK |
			UART0_C1_DOZEEN_MASK |
			UART0_C1_RSRC_MASK |
			UART0_C1_M_MASK |                 
			UART0_C1_WAKE_MASK |
			UART0_C1_ILT_MASK |
			UART0_C1_PE_MASK |			
			UART0_C1_PT_MASK
	);

	UART0_C1 |= (config->c1_loops << UART0_C1_LOOPS_SHIFT |
			config->c1_dozeen << UART0_C1_DOZEEN_SHIFT |
			config->c1_rsrc << UART0_C1_RSRC_SHIFT |
			config->c1_m << UART0_C1_M_SHIFT |                 
			config->c1_wake << UART0_C1_WAKE_SHIFT |
			config->c1_ilt << UART0_C1_ILT_SHIFT |
			config->c1_pe << UART0_C1_PE_SHIFT |			
			config->c1_pt << UART0_C1_PT_SHIFT
	);

	// Configurar o registrador C2
	UART0_C2 &= ~(UART0_C2_RWU_MASK |
			UART0_C2_SBK_MASK);

	UART0_C2 |= (config->c2_rwu << UART0_C2_RWU_SHIFT |
			config->c2_sbk << UART0_C2_SBK_SHIFT
	);

	// Configurar o registrador S2
	UART0_S2 &= ~(UART0_S2_MSBF_MASK |
			UART0_S2_RXINV_MASK |
			UART0_S2_RWUID_MASK |
			UART0_S2_BRK13_MASK |
			UART0_S2_LBKDE_MASK
	);

	UART0_S2 |= (config->s2_msbf << UART0_S2_MSBF_SHIFT |
			config->s2_rxinv << UART0_S2_RXINV_SHIFT |
			config->s2_rwuid << UART0_S2_RWUID_SHIFT |
			config->s2_brk13 << UART0_S2_BRK13_SHIFT |
			config->s2_lbkde << UART0_S2_LBKDE_SHIFT
	);

	// Configurar o registrador C3
	UART0_C3 &= ~(UART0_C3_R8T9_MASK |
			UART0_C3_R9T8_MASK |
			UART0_C3_TXDIR_MASK |
			UART0_C3_TXINV_MASK 
	);

	UART0_C3 |= (config->c3_r8t9 << UART0_C3_R8T9_SHIFT |
			config->c3_r9t8 << UART0_C3_R9T8_SHIFT |
			config->c3_txdir << UART0_C3_TXDIR_SHIFT |
			config->c3_txinv << UART0_C3_TXINV_SHIFT 
	);

	// Configurar o registrador C4
	UART0_C4 &= ~(UART0_C4_MAEN1_MASK |
			UART0_C4_MAEN2_MASK |
			UART0_C4_M10_MASK
	);

	UART0_C4 |= (config->c4_maen1 << UART0_C4_MAEN1_SHIFT |
			config->c4_maen2 << UART0_C4_MAEN2_SHIFT |
			config->c4_m10 << UART0_C4_M10_SHIFT
	);

	// Valores fora do intervalo 4 a 32 implicam em setar 16
	UART0_C4 |= (0b11111 << UART0_C4_OSR_SHIFT);
	UART0_C4 &= ((config->c4_osr << UART0_C4_OSR_SHIFT) | ~UART0_C4_OSR_MASK);

	// Configurar o registrador C5				
	UART0_C5 &= ~(UART0_C5_TDMAE_MASK |
			UART0_C5_RDMAE_MASK |
			UART0_C5_BOTHEDGE_MASK |
			UART0_C5_RESYNCDIS_MASK
	);

	UART0_C5 |= (config->c5_tdmae << UART0_C5_TDMAE_SHIFT |
			config->c5_rdmae << UART0_C5_RDMAE_SHIFT |
			config->c5_bothedge << UART0_C5_BOTHEDGE_SHIFT |
			config->c5_resyncdis << UART0_C5_RESYNCDIS_SHIFT
	);

	// Ajustar amostragem conforme a taxa de superamostragem
	if ((config->c4_osr & 0b11111) > 2 && (config->c4_osr & 0b11111) < 7)
		UART0_C5 |= UART0_C5_BOTHEDGE_MASK;    

	// Mascarar bits invalidos
	config->sbr |= (config->sbr & 0x1FFF);

	// Setar baus rate via SBR
	UART0_BDH &= ~UART0_BDH_SBR(0x1F);                             
	UART0_BDL &= ~UART0_BDL_SBR(0xFF);            
	UART0_BDH |= UART0_BDH_SBR(config->sbr >> 8);                             
	UART0_BDL |= UART0_BDL_SBR(config->sbr & 0x00FF);            

	//Habilitar os canais RX e TX
	UART0_C2 |= (UART0_C2_RE_MASK |         		///< habilita o canal receptor 
			UART0_C2_TE_MASK);	  			///< habilita o canal transmissor  

	return 0;
}

/**
 * @brief Reconfigurar UART0
 */
void UART0_reconfig(UART0Config_type *config) {
	// Desabilitar canais RX e TX para configuracoes
	UART0_C2 &= ~(UART0_C2_RE_MASK |           
			UART0_C2_TE_MASK);

	// Configurar stop bits
	UART0_BDH &= ~(UART0_BDH_SBNS_MASK);
	UART0_BDH |= config->bdh_sbns << UART0_BDH_SBNS_SHIFT;

	// Configurar o registrador C1
	UART0_C1 &= ~(UART0_C1_LOOPS_MASK |
			UART0_C1_DOZEEN_MASK |
			UART0_C1_RSRC_MASK |
			UART0_C1_M_MASK |                 
			UART0_C1_WAKE_MASK |
			UART0_C1_ILT_MASK |
			UART0_C1_PE_MASK |			
			UART0_C1_PT_MASK
	);

	UART0_C1 |= (config->c1_loops << UART0_C1_LOOPS_SHIFT |
			config->c1_dozeen << UART0_C1_DOZEEN_SHIFT |
			config->c1_rsrc << UART0_C1_RSRC_SHIFT |
			config->c1_m << UART0_C1_M_SHIFT |                 
			config->c1_wake << UART0_C1_WAKE_SHIFT |
			config->c1_ilt << UART0_C1_ILT_SHIFT |
			config->c1_pe << UART0_C1_PE_SHIFT |			
			config->c1_pt << UART0_C1_PT_SHIFT
	);

	// Configurar o registrador C2
	UART0_C2 &= ~(UART0_C2_RWU_MASK |
			UART0_C2_SBK_MASK);

	UART0_C2 |= (config->c2_rwu << UART0_C2_RWU_SHIFT |
			config->c2_sbk << UART0_C2_SBK_SHIFT
	);

	// Configurar o registrador S2
	UART0_S2 &= ~(UART0_S2_MSBF_MASK |
			UART0_S2_RXINV_MASK |
			UART0_S2_RWUID_MASK |
			UART0_S2_BRK13_MASK |
			UART0_S2_LBKDE_MASK
	);

	UART0_S2 |= (config->s2_msbf << UART0_S2_MSBF_SHIFT |
			config->s2_rxinv << UART0_S2_RXINV_SHIFT |
			config->s2_rwuid << UART0_S2_RWUID_SHIFT |
			config->s2_brk13 << UART0_S2_BRK13_SHIFT |
			config->s2_lbkde << UART0_S2_LBKDE_SHIFT
	);

	// Configurar o registrador C3
	UART0_C3 &= ~(UART0_C3_R8T9_MASK |
			UART0_C3_R9T8_MASK |
			UART0_C3_TXDIR_MASK |
			UART0_C3_TXINV_MASK 
	);

	UART0_C3 |= (config->c3_r8t9 << UART0_C3_R8T9_SHIFT |
			config->c3_r9t8 << UART0_C3_R9T8_SHIFT |
			config->c3_txdir << UART0_C3_TXDIR_SHIFT |
			config->c3_txinv << UART0_C3_TXINV_SHIFT 
	);

	// Configurar o registrador C4
	UART0_C4 &= ~(UART0_C4_MAEN1_MASK |
			UART0_C4_MAEN2_MASK |
			UART0_C4_M10_MASK
	);

	UART0_C4 |= (config->c4_maen1 << UART0_C4_MAEN1_SHIFT |
			config->c4_maen2 << UART0_C4_MAEN2_SHIFT |
			config->c4_m10 << UART0_C4_M10_SHIFT
	);

	// Valores fora do intervalo 4 a 32 implicam em setar 16
	UART0_C4 |= (0b11111 << UART0_C4_OSR_SHIFT);
	UART0_C4 &= ((config->c4_osr << UART0_C4_OSR_SHIFT) | ~UART0_C4_OSR_MASK);

	// Configurar o registrador C5				
	UART0_C5 &= ~(UART0_C5_TDMAE_MASK |
			UART0_C5_RDMAE_MASK |
			UART0_C5_BOTHEDGE_MASK |
			UART0_C5_RESYNCDIS_MASK
	);

	UART0_C5 |= (config->c5_tdmae << UART0_C5_TDMAE_SHIFT |
			config->c5_rdmae << UART0_C5_RDMAE_SHIFT |
			config->c5_bothedge << UART0_C5_BOTHEDGE_SHIFT |
			config->c5_resyncdis << UART0_C5_RESYNCDIS_SHIFT
	);

	// Ajustar amostragem conforme a taxa de superamostragem
	if ((config->c4_osr & 0b11111) > 2 && (config->c4_osr & 0b11111) < 7)
		UART0_C5 |= UART0_C5_BOTHEDGE_MASK;    

	// Mascarar bits invalidos
	config->sbr |= (config->sbr & 0x1FFF);

	// Setar baus rate via SBR
	UART0_BDH &= ~UART0_BDH_SBR(0x1F);                             
	UART0_BDL &= ~UART0_BDL_SBR(0xFF);            
	UART0_BDH |= UART0_BDH_SBR(config->sbr >> 8);                             
	UART0_BDL |= UART0_BDL_SBR(config->sbr & 0x00FF);            

	//Habilitar os canais RX e TX
	UART0_C2 |= (UART0_C2_RE_MASK |         		///< habilita o canal receptor 
			UART0_C2_TE_MASK);	  			///< habilita o canal transmissor  

	return;
}

/**
 * @brief Habilitar os bits de interrupes
 * @param[in] config bits de donfiguracao 
 */
void UART0_configInterrupts (UART0ConfigInterrupts_type *config) {
	UART0_C2 &= ~(UART0_C2_TIE_MASK |
			UART0_C2_TCIE_MASK |
			UART0_C2_RIE_MASK |
			UART0_C2_ILIE_MASK);

	UART0_C2 |= (config->c2_tie << UART0_C2_TIE_SHIFT |
			config->c2_tcie << UART0_C2_TCIE_SHIFT |
			config->c2_rie << UART0_C2_RIE_SHIFT |
			config->c2_ilie << UART0_C2_ILIE_SHIFT);

	UART0_C3 &= ~(UART0_C3_ORIE_MASK |
			UART0_C3_NEIE_MASK |
			UART0_C3_FEIE_MASK |
			UART0_C3_PEIE_MASK);

	UART0_C3 |= (config->c3_orie << UART0_C3_ORIE_SHIFT |
			config->c3_neie << UART0_C3_NEIE_SHIFT |
			config->c3_feie << UART0_C3_FEIE_SHIFT |
			config->c3_peie << UART0_C3_PEIE_SHIFT);
}

/**
 * @brief Habilitar a interrupcao do canal receptor
 */
void UART0_enableRIEInterrup (void) {
	UART0_C2 |= UART_C2_RIE_MASK;	
}

/**
 * @brief Habilitar a interrupcao do canal transmissor
 */
void UART0_enableTEInterrup (void) {
	UART0_C2 |= UART_C2_TIE_MASK;
}

/**
 * @brief Desabilitar a interrupcao do canal receptor
 */
void UART0_disableRIEInterrup (void) {
	UART0_C2 &= ~UART_C2_RIE_MASK;
}

/**
 * @brief Desabilitar a interrupcao do canal transmissor
 */
void UART0_disableTEInterrup (void) {
	UART0_C2 &= ~UART_C2_TIE_MASK;
}

/**
 * @brief Habilitar o canal receptor
 */
void UART0_enableRE (void) {
	UART0_C2 |= UART_C2_RE_MASK;
}

/**
 * @brief Habilitar o canal transmissor
 */
void UART0_enableTE (void) {
	UART0_C2 |= UART_C2_TE_MASK;
}

/**
 * @brief Desabilitar o canal recpetor
 */
void UART0_disableRE (void) {
	UART0_C2 &= ~UART_C2_RE_MASK;
}

void disableTE (void) {
	UART0_C2 &= ~UART_C2_TE_MASK;	
}

//=====================================
// NVIC
//=====================================
void NVIC_enableUART0(char priority)
{
	NVIC_ISER |= 1 << (28-16) ;      		// Habilitar IRQ12
	NVIC_ICPR |= 1 << (28-16) ;      		// Limpar as pendencias em solicitacao de interrupcoes
	NVIC_IPR3 |= NVIC_IP_PRI_12(priority << 6); ///< seta prioridade 		
}

//=====================================
// Estrutura de dados: buffer circular
//=====================================
typedef struct BufferCirc_tag
{
	char * dados;  	 // buffer de dados
	unsigned int tamanho;	// quantidade total de elementos
	unsigned int leitura;       // indice de leitura (tail)  
	unsigned int escrita;       // indice de escrita (head)
} BufferCirc_type;

/**
 * @brief Inicializar um buffer circular
 * @param[in] buffer buffer circular
 * @param[in] tamanho tamanho do buffer circular
 */
void BC_init(BufferCirc_type *buffer, unsigned int tamanho) 
{
	buffer->dados = malloc(tamanho * sizeof(char));
	buffer->tamanho = tamanho;
	buffer->escrita = 0;
	buffer->leitura = 0;
}

/**
 * @brief Resetar um beffer circular
 * @param[in] buffer buffer circular
 */
void BC_reset(BufferCirc_type *buffer) 
{
	buffer->escrita = buffer->leitura = 0;
}

/**
 * @brief Liberar um buffer circular
 * @param[in] buffer buffer circular
 */
void BC_free(BufferCirc_type *buffer)
{
	buffer->tamanho = 0;
	buffer->escrita = 0;
	buffer->leitura = 0;
	//Liberar o espaco de memoria ocupado pelos dados
	free(buffer->dados);
}

/**
 * @brief Inserir um elemento no buffer circular
 * @param[in] buffer buffer circular
 * @param[in] item elemento
 */
int BC_push (BufferCirc_type *buffer, char item)
{
	unsigned int next;

	next = buffer->escrita+1;
	if (next == buffer->tamanho) next = 0;

	if (next == buffer->leitura) {
		//sem espaco no buffer
		return -1;
	}
	buffer->dados[buffer->escrita] = item;
	buffer->escrita = next;

	return 0;
}

/**
 * @brief Remover um elemento no buffer circular
 * @param[in] buffer buffer circular
 * @param[out] item elemento
 */
int BC_pop (BufferCirc_type *buffer, char *item)
{
	unsigned int next;

	if (buffer->escrita == buffer->leitura) {
		//buffer vazio
		buffer->escrita = buffer->leitura = 0; //reset
		return -1;
	}
	next = buffer->leitura + 1;
	if (next == buffer->tamanho) next = 0;
	*item = buffer->dados[buffer->leitura];
	buffer->leitura = next;	

	return 0;
}

/**
 * @brief Computar a quantidade de elementos no buffer
 * @return quantidade de elementos
 */
unsigned int BC_elements (BufferCirc_type *buffer)
{ 
	unsigned int tmp;
	if (buffer->escrita > buffer->leitura) tmp = buffer->escrita - buffer->leitura;
	else tmp = (buffer->tamanho - buffer->leitura) + buffer->escrita;

	return tmp;
}

/**
 * @brief Estado do buffer cheio?
 */
uint8_t BC_isFull (BufferCirc_type *buffer)
{
	unsigned int next = buffer->escrita+1;
	if (next == buffer->tamanho) next = 0;

	if (next == buffer->leitura) {
		//sem espaco no buffer
		return 1;
	}

	return 0;
}

/**
 * @brief Estado do buffer vazio?
 */
uint8_t BC_isEmpty (BufferCirc_type *buffer)
{
	if (buffer->leitura == buffer->escrita) {
		return 1;
	}
	return 0;
}

//=====================================
// Application
//=====================================
UART0Config_type config = {
		.bdh_sbns=0,			// Um stop bit
		.sbr=0b100, 			// MCGFLLCLK/(baud_rate*osr)=4
		.c1_loops=0,			// Operacao normal: full-duplex 
		.c1_dozeen=0,			// UART habilitado no modo WAIT
		.c1_rsrc=0,				// nao tem efeito para loops == 0
		.c1_m=0,				// dados em 8 bits
		.c1_wake=0,				// wakeup de RX por linha ociosa (IDLE)
		.c1_ilt=0,				// deteccao do estado ocioso pela contagem apos start bits
		.c1_pe=0,				// nao ha checagem de paridade por hardware
		.c1_pt=0,				// nao tem efeito para pe==0
		.c2_rwu=0,				// operacao normal de recepcao do UART0
		.c2_sbk=0,				// operacao normal de transmissor do UART0
		.s2_msbf=0,				// envio de LSB para MSB
		.s2_rxinv=0,			// polaridade dos dados de RX nao eh invertida
		.s2_rwuid=0,			// nao tem efeito para rwu==0
		.s2_brk13=0,			// nao tem efeito para operacao normal de transmissor
		.s2_lbkde=0,			// deteccao do caractere break no tamanho menor
		.c3_r8t9=0,				// nao tem efeito para m==0
		.c3_r9t8=0,				// nao tem efeito para m==0
		.c3_txdir=0,			// nao tem efeito para loops==0
		.c3_txinv=0,			// polaridade dos dados de TX nao eh invertida 
		.c4_maen1=0,			// todos os dados recebidos sao transferidos para buffer de dados
		.c4_maen2=0,			
		.c4_m10=0,				// tamanho dos dados em RX e TX  8 ou 9 bits
		.c4_osr=0b01111,		// superamostragem 16x
		.c5_tdmae=0,			// uso de DMA pelo TX desabilitada
		.c5_rdmae=0,			// uso de DMA pelo RX desabilitado
		.c5_bothedge=0,			// amostragem somente na borda de subida do clock
		.c5_resyncdis=0			// resincronizacao na recepcao habilitada
};

UART0ConfigInterrupts_type configInterrupt = {
		.c2_tie = 0,		// desabilitar interrupcao TX
		.c2_tcie = 0,		// desabilitar interrupcao completa TX
		.c2_rie = 0, 		// desabilitar interrupcao do receptor
		.c2_ilie = 0, 		// desabilitar interrupcao por linha ociosa
		.c3_orie = 0, 		// desabilitar interrupcao por overrun
		.c3_neie = 0, 		// desabilitar interrupcao por noise error
		.c3_feie = 0,		// desabilitar interrupcao por framing error
		.c3_peie = 0		// desabilitar interrupcao por parity error
};

BufferCirc_type buffer;

void UART0_IRQHandler()
{
	char item;

	if (UART0_S1 & UART0_S1_TDRE_MASK) {
		BC_pop (&buffer, &item);
		UART0_D = item;
	} 
}

// Baud rates: 300, 1200, 2400, 4800, 9600, 19200, 38400, 57600, 115200
#define BAUD_RATE 9600
#define TAM_MAX 128
int main (void)
{
	SIM_initBusClock (0b000);	// Divisor de frequencia em 2
	SIM_selPLLFLL (0); 			// selecionar MCGFLLCLK (20971520Hz)
	PORTx_muxUART0RxTxPins();

	NVIC_enableUART0(2);		// habilitar IRQ12

	// Configurar UART0
	config.c4_osr = 12;
	config.sbr = (uint16_t)(20971520./(BAUD_RATE * (config.c4_osr+1)));
	if ((20971520./(1.0 * BAUD_RATE * (config.c4_osr+1))) > config.sbr)
		config.sbr++;
	config.c1_pe=1;				// nao ha checagem de paridade por hardware
	config.c1_pt=1;				// paridade impar
	config.c1_m=1;				// bit de paridade eh 9o. bit
	UART0_init(&config);

	NVIC_enableUART0(2);		// habilitar IRQ12

	BC_init(&buffer, TAM_MAX);

	uint32_t i = 0;
	BC_push( &buffer, 0x30+(i));

	// Habilitar interrupcoes
	configInterrupt.c2_tie = 1;			// habilitar TX interrupt
	UART0_configInterrupts (&configInterrupt);

	for(;;) {
		if (!(UART0_S1 & UART0_S1_TDRE_MASK)) {
			// Inserir caracteres ASCII a partir de 0x30
			i++;
			if (i == 75) {
				i = 0;
				while (BC_push( &buffer, '\n')==-1); //newline
				while (BC_push( &buffer, '\r')==-1); //carriage return
			}
			while (BC_push( &buffer, 0x30+(i))==-1);

		}
	}	
	return 0;
}
